<?php

namespace App\Http\Controllers;

use App\Models\Invoice;
use App\Models\Location;
use App\Models\LocationState;
use App\Models\Message;
use App\Models\MessageType;
use App\Models\Subscription;
use App\Models\User;
use App\Models\UserType;
use Illuminate\Contracts\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class InvoiceController extends Controller
{
    public function list(array $extras = []) {
        $invoices = Invoice::where('user_id', Auth::user()->id)->orderBy('created_at', 'desc')->paginate(10);
        return view('pages.invoice', array_merge(['invoices' => $invoices], $extras));
    }

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $users = User::where('type', UserType::MEMBER)->where('active', true)->get();
        $locations = Location::where('active', true)->where('state', LocationState::AVAILABLE)->get();

        return $this->list(['users'=> $users, 'locations'=> $locations]);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $messages = array();

        $validator = Validator::make($request->all(), [
            'userId' => ['required','string'],
            'locationId' => ['required','string'],
            'duration' => ['required','string'],
            'durationUnit' => ['required','in:DAY,HOUR']
        ]);

        if ($validator->fails()) {
            foreach ($validator->errors() as $error) {
                array_push($messages, new Message($error, MessageType::ERROR));
            }
        } else {
            try {
                $expiryDate = date_add(new \DateTime(), date_interval_create_from_date_string($request->duration.' '.strtolower($request->durationUnit)));

                array_push($messages, new Message($request->duration.$request->durationUnit.$request->userId.$request->locationId, MessageType::INFO));

                $subscription = Subscription::create([
                    'user_id' => $request->userId,
                    'location_id' => $request->locationId,
                    'duration' => $request->duration,
                    'duration_unit' => $request->durationUnit,
                    'active' => true,
                    'expiry_date' => $expiryDate,
                ]);

                // Setting location as occupied
                $location = Location::find($request->locationId);

                if (!$location) {
                    throw new \Error('Cannot find location with id: '.$request->locationId);
                }

                $location->state = LocationState::OCCUPIED;
                $location->save();

                $invoice = Invoice::create([
                    'user_id' => Auth::user()->id,
                    'subscription_id' => $subscription->id,
                    'paid' => false
                ]);

                array_push($messages, new Message('Location created successfully with the ID: '.$invoice->id, MessageType::SUCCESS));
            } catch (\Error $error) {
                array_push($messages, new Message('Failed with error: '.$error->getMessage(), MessageType::ERROR));
            }
        }

        $users = User::where('type', 'MEMBER');
        $locations = Location::where('active', true)->where('state', LocationState::AVAILABLE)->get();

        return $this->list(['messages' => $messages, 'users' => $users, 'locations'=> $locations]);
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Request $request, string $invoiceId)
    {
        return view('pages.test', ['requestUrl' => $request->session()->previousUrl()]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Invoice $invoice)
    {
        //
    }

    public function togglePaid(Request $request, string $invoiceId) {
        $messages = array();

        $invoice = Invoice::find($invoiceId);

        if (!$invoice) {
            array_push($messages, new Message('User with id '.$invoiceId.' not found in database', MessageType::ERROR));
        } else {
            try {
                $invoice->paid = !$invoice->paid;
                $messageState = $invoice->paid ? 'Paid': 'Unpaid';

                $invoice->save();

                array_push($messages, new Message('Marked invoice with id '.$invoiceId.' as '.$messageState, MessageType::SUCCESS));
            } catch (\Error $error) {
                array_push($messages, new Message('Failed with error: '.$error->getMessage(), MessageType::ERROR));
            }
        }

        $urlComponents =  explode('?', $request->session()->previousUrl());
        $url = '/invoices';

        if (count($urlComponents) > 1 && $urlComponents[1]) {
            $url .= '?'.$urlComponents[1];
        }

        return redirect($url)->with('messages', $messages);
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Invoice $invoice)
    {
        //
    }

    public function search(Request $request) {
        $messages = array();

        $keyword = $request->input('keyword');

        if (!$keyword || $keyword == '') {
            array_push($messages, new Message('Keyword for search cannot be empty', MessageType::ERROR));
        }

        $invoices = Invoice::whereHas('user', function (Builder $query) use ( $keyword ) {
            $query->where('name', 'like', '%' . $keyword . '%')
                ->orWhere('phone', 'like', '%' . $keyword . '%');
        })->paginate(10);

        if (!$invoices) {
            array_push($messages, new Message('No search results found!', MessageType::ERROR));
            return redirect('/invoices')->with('messages', $messages);
        }

        $users = User::where('type', 'MEMBER');
        $locations = Location::where('active', true)->where('state', LocationState::AVAILABLE)->get();

        return view('pages.invoice', ['invoices'=> $invoices, 'users' => $users, 'locations' => $locations, 'messages'=> $messages]);
    }
}
