<?php

namespace App\Http\Controllers;

use App\Models\Location;
use App\Models\Message;
use App\Models\MessageType;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

use File;

class LocationController extends Controller
{
    public function list(array $extras = []) {
        $locations = Location::orderBy('created_at', 'desc')->paginate(10);
        return view('pages.location', array_merge(['locations' => $locations], $extras));
    }

    // Show locations
    public function index() {
        return $this->list();
    }

    // Create location
    public function store(Request $request) {
        $messages = array();

        $validator = Validator::make($request->all(), [
            'title' => ['required', 'string'],
            'type' => ['required', 'in:DESK,STORAGE,ROOM'],
            'address' => ['required', 'string'],
            'image' => ['required', 'mimes:png,jpg,jpeg', 'max:5120', 'min:1']
        ]);

        if ($validator->fails()) {
            foreach ($validator->errors() as $error) {
                array_push($messages, new Message($error, MessageType::ERROR));
            }
        } else {
            try {
                if (!$request->rate_hourly && !$request->rate_daily) {
                    throw new \Error("All rates cannot be empty");
                }

                $image = $request->type.time().'.'.$request->image->extension();

                $location = Location::create([
                    'title' => $request->title,
                    'type' => $request->type,
                    'description' => $request->description,
                    'address' => $request->address,
                    'latitude' => floatval($request->latitude),
                    'longitude' =>floatval($request->longitude),
                    'rate_hourly' => $request->rate_hourly,
                    'rate_daily' => $request->rate_daily,
                    'image' => $image
                ]);

                $request->image->storeAs('images/products', $image);

                array_push($messages, new Message('Location created successfully with the ID: '.$location->id, MessageType::SUCCESS));
            } catch (\Error $error) {
                array_push($messages, new Message('Failed with error: '.$error->getMessage(), MessageType::ERROR));
            }
        }

        return $this->list(['messages' => $messages]);
    }

    public function edit(Request $request, string $locationId) {
        $location = Location::find($locationId);

        if (!$location) {
            return redirect('/locations');
        }

        return $this->list(['selectedLocation' => $location]);
    }

    public function update(Request $request, string $locationId) {
        $messages = array();

        $validator = Validator::make($request->all(), [
            'title' => ['required', 'string'],
            'description' => ['string'],
            'address' => ['required', 'string'],
            'image' => ['mimes:png,jpg,jpeg', 'max:5120', 'min:1']
        ]);

        if($validator->fails()) {
            foreach ($validator->errors() as $error) {
                array_push($messages, new Message($error, MessageType::ERROR));
            }
        } else {
            $location = Location::find($locationId);

            if (!$location) {
                array_push($messages, new Message('Location with id '.$locationId.' not found in database', MessageType::ERROR));
            } else {
                try {
                    if (!$request->rate_hourly && !$request->rate_daily) {
                        throw new \Error("All rates cannot be empty");
                    }

                    $location->title = $request->title;
                    $location->description = $request->description;
                    $location->address = $request->address;
                    $location->latitude = floatval($request->longitude);
                    $location->longitude = floatval($request->latitude);
                    $location->rate_hourly = $request->rate_hourly;
                    $location->rate_daily = $request->rate_daily;

                    if ($request->image) {
                        $oldImagePath = public_path('/images/products'.$location->image);
                        // Delete existing image
                        if (File::exists($oldImagePath)) {
                            File::delete($oldImagePath);
                        }

                        $image = $request->type.time().'.'.$request->image->extension();
                        $location->image = $image;

                        $request->image->storeAs('images/products', $image);
                    }

                    $location->save();

                    array_push($messages, new Message('Updated location '.$request->title, MessageType::SUCCESS));
                } catch (\Error $error) {
                    array_push($messages, new Message('Failed with error: '.$error->getMessage(), MessageType::ERROR));
                }
            }
        }

        return redirect('/locations')->with('messages', $messages);
    }

    public function toggleActivation(Request $request, string $locationId) {
        $messages = array();

        $location = Location::find($locationId);

        if (!$location) {
            array_push($messages, new Message('User with id '.$locationId.' not found in database', MessageType::ERROR));
        } else {
            try {
                $location->active = !$location->active;
                $messageState = $location->active ? 'Activated': 'Deactivated';

                $location->save();

                array_push($messages, new Message($messageState.' location with id '.$locationId, MessageType::SUCCESS));
            } catch (\Error $error) {
                array_push($messages, new Message('Failed with error: '.$error->getMessage(), MessageType::ERROR));
            }
        }

        $urlComponents =  explode('?', $request->session()->previousUrl());
        $url = '/locations';

        if (count($urlComponents) > 1 && $urlComponents[1]) {
            $url .= '?'.$urlComponents[1];
        }

        return redirect($url)->with('messages', $messages);
    }

    public function search(Request $request) {
        $messages = array();

        $keyword = $request->input('keyword');

        if (!$keyword || $keyword == '') {
            array_push($messages, new Message('Keyword for search cannot be empty', MessageType::ERROR));
        }

        $locations = Location::where('title', 'like', '%' . $keyword . '%')
            ->orWhere('address', 'like', '%' . $keyword . '%')
            ->paginate(10);

        if (!$locations) {
            array_push($messages, new Message('No search results found!', MessageType::ERROR));
            return redirect('/locations')->with('messages', $messages);
        }

        return view('pages.location', ['locations'=> $locations, 'messages'=> $messages]);
    }
}
