<?php

namespace App\Http\Controllers;

use App\Models\Message;
use App\Models\MessageType;
use App\Models\User;
use App\Models\UserType;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;


class UserController extends Controller
{
    public function list(array $extras = []) {
        $currentUserId = Auth::user()->id;
        $users = User::where('id', '!=', $currentUserId)->paginate(10);

        return view('pages.user', array_merge(['users' => $users], $extras));
    }

    public function index(array $messages = []) {
        return $this->list($messages);
    }

    public function edit(Request $request, string $userId) {
        $user = User::find($userId);

        if (!$user) {
            return redirect('/users');
        }

        return $this->list(['selectedUser' => $user]);
    }

    public function update(Request $request, string $userId) {
        $messages = array();

        $validator = Validator::make($request->all(), [
            'type' => ['required', 'in:MEMBER,STAFF,ADMIN']
        ]);

        if ($validator->fails()) {
            foreach ($validator->errors() as $error) {
                array_push($messages, new Message($error, MessageType::ERROR));
            }
        } else {
            $user = User::find($userId);

            if (!$user) {
                array_push($messages, new Message('User with id ' . $userId . ' not found in database', MessageType::ERROR));
            } else {
                try {
                    $user->type = match ($request->type) {
                        'MEMBER' => UserType::MEMBER,
                        'STAFF' => UserType::STAFF,
                        'ADMIN' => UserType::ADMIN
                    };

                    $user->save();

                    array_push($messages, new Message('Updated user type to ' . $request->type . ' for user ' . $userId, MessageType::SUCCESS));
                } catch (\Error $error) {
                    array_push($messages, new Message('Failed with error: ' . $error->getMessage(), MessageType::ERROR));
                }
            }
        }

        return redirect('/users')->with('messages', $messages);
    }

    public function toggleActivation(Request $request, string $userId) {
        $messages = array();

        $user = User::find($userId);

        if (!$user) {
            array_push($messages, new Message('User with id ' . $userId . ' not found in database', MessageType::ERROR));
        } else {
            try {
                $user->active = !$user->active;
                $messageState = $user->active ? 'Activated' : 'Deactivated';

                $user->save();

                array_push($messages, new Message($messageState . ' user ' . $user->name . ' with id ' . $userId, MessageType::SUCCESS));
            } catch (\Error $error) {
                array_push($messages, new Message('Failed with error: ' . $error->getMessage(), MessageType::ERROR));
            }
        }

        $urlComponents = explode('?', $request->session()->previousUrl());
        $url = str_replace(route('home'),'', $urlComponents[0]);

        if (count($urlComponents) > 1 && $urlComponents[1]) {
            $url .= '?' . $urlComponents[1];
        }

        return redirect($url)->with('messages', $messages);
    }

    public function search(Request $request) {
        $messages = array();

        $keyword = $request->input('keyword');

        if (!$keyword || $keyword == '') {
            array_push($messages, new Message('Keyword for search cannot be empty', MessageType::ERROR));
        }

        $users = User::where('name', 'like', '%' . $keyword . '%')
            ->orWhere('phone', 'like', '%' . $keyword . '%')
            ->orWhere('email', 'like', '%' . $keyword . '%')
            ->paginate(10);

        if (!$users) {
            array_push($messages, new Message('No search results found!', MessageType::ERROR));
            return redirect('/users')->with('messages', $messages);
        }

        return view('pages.user', ['users'=> $users, 'messages'=> $messages]);
    }
}
